@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1991, 1997-2000,2012 Free Software Foundation, Inc.
+** Copyright (C) 1991, 1997-2017,2019 Free Software Foundation, Inc.
 **
 ** This file is part of TACK.
 **
@@ -21,58 +21,30 @@
 
 #include <tack.h>
 
-MODULE_ID("$Id: charset.c,v 1.15 2012/04/29 00:14:39 tom Exp $")
+MODULE_ID("$Id: charset.c,v 1.26 2019/07/21 18:14:30 tom Exp $")
 
 /*
 	Menu definitions for alternate character set and SGR tests.
 */
 
-static void charset_bel(struct test_list *t, int *state, int *ch);
-static void charset_flash(struct test_list *t, int *state, int *ch);
-static void charset_civis(struct test_list *t, int *state, int *ch);
-static void charset_cvvis(struct test_list *t, int *state, int *ch);
-static void charset_cnorm(struct test_list *t, int *state, int *ch);
-static void charset_hs(struct test_list *t, int *state, int *ch);
-static void charset_eslok(struct test_list *t, int *state, int *ch);
-static void charset_status(struct test_list *t, int *state, int *ch);
-static void charset_dsl(struct test_list *t, int *state, int *ch);
-static void charset_enacs(struct test_list *t, int *state, int *ch);
-static void charset_smacs(struct test_list *t, int *state, int *ch);
-static void charset_attributes(struct test_list *t, int *state, int *ch);
-static void charset_sgr(struct test_list *t, int *state, int *ch);
-
-struct test_list acs_test_list[] =
-{
-    {0, 0, 0, 0, "e) edit terminfo", 0, &edit_menu},
-    {MENU_NEXT, 3, "bel", 0, 0, charset_bel, 0},
-    {MENU_NEXT, 3, "flash", 0, 0, charset_flash, 0},
-    {MENU_NEXT, 3, "civis", 0, 0, charset_civis, 0},
-    {MENU_NEXT, 3, "cvvis", 0, 0, charset_cvvis, 0},
-    {MENU_NEXT, 3, "cnorm", 0, 0, charset_cnorm, 0},
-    {MENU_NEXT, 3, "hs", 0, 0, charset_hs, 0},
-    {MENU_NEXT, 3, "tsl) (fsl) (wsl", "hs", 0, charset_status, 0},
-    {MENU_NEXT, 3, "eslok", "hs", 0, charset_eslok, 0},
-    {MENU_NEXT, 3, "dsl", "hs", 0, charset_dsl, 0},
-    {MENU_NEXT, 0, "acsc) (enacs) (smacs) (rmacs", 0, 0, charset_enacs, 0},
-    {MENU_NEXT, 0, "smacs) (rmacs", 0, 0, charset_smacs, 0},
-    {MENU_NEXT, 11, 0, 0, 0, charset_attributes, 0},
-    {MENU_NEXT, 11, "sgr) (sgr0", "ma", 0, charset_sgr, 0},
-    {MENU_LAST, 0, 0, 0, 0, 0, 0}
-};
+#undef  BIT
+#define BIT(n) (1 << (n))
+/* *INDENT-OFF* */
 
 const struct mode_list alt_modes[] =
 {
-    {"normal", "(sgr0)", "(sgr0)", 1},
-    {"standout", "(smso)", "(rmso)", 2},
-    {"underline", "(smul)", "(rmul)", 4},
-    {"reverse", "(rev)", "(sgr0)", 8},
-    {"blink", "(blink)", "(sgr0)", 16},
-    {"dim", "(dim)", "(sgr0)", 32},
-    {"bold", "(bold)", "(sgr0)", 64},
-    {"invis", "(invis)", "(sgr0)", 128},
-    {"protect", "(prot)", "(sgr0)", 256},
-    {"altcharset", "(smacs)", "(rmacs)", 512}
+    {"normal",     "(sgr0)",  "(sgr0)",  BIT(0)},
+    {"standout",   "(smso)",  "(rmso)",  BIT(1)},
+    {"underline",  "(smul)",  "(rmul)",  BIT(2)},
+    {"reverse",    "(rev)",   "(sgr0)",  BIT(3)},
+    {"blink",      "(blink)", "(sgr0)",  BIT(4)},
+    {"dim",        "(dim)",   "(sgr0)",  BIT(5)},
+    {"bold",       "(bold)",  "(sgr0)",  BIT(6)},
+    {"invis",      "(invis)", "(sgr0)",  BIT(7)},
+    {"protect",    "(prot)",  "(sgr0)",  BIT(8)},
+    {"altcharset", "(smacs)", "(rmacs)", BIT(9)},
 };
+/* *INDENT-ON* */
 
 /* On many terminals the underline attribute is the last scan line.
    This is OK unless the following line is reverse video.
@@ -131,7 +103,7 @@ static struct graphics_pair glyph[] =
 */
 static void
 charset_hs(
-	      struct test_list *t,
+	      TestList * t,
 	      int *state,
 	      int *ch)
 {
@@ -148,7 +120,7 @@ charset_hs(
 */
 static void
 charset_status(
-		  struct test_list *t,
+		  TestList * t,
 		  int *state,
 		  int *ch)
 {
@@ -182,7 +154,7 @@ charset_status(
 */
 static void
 charset_eslok(
-		 struct test_list *t,
+		 TestList * t,
 		 int *state,
 		 int *ch)
 {
@@ -227,7 +199,7 @@ charset_eslok(
 */
 static void
 charset_dsl(
-	       struct test_list *t,
+	       TestList * t,
 	       int *state,
 	       int *ch)
 {
@@ -253,7 +225,7 @@ eat_cookie(void)
 }
 
 void
-put_mode(char *s)
+put_mode(const char *s)
 {				/* send the attribute string (with or without
 				   % execution) */
     tc_putp(TPARM_0(s));	/* allow % execution */
@@ -262,7 +234,8 @@ put_mode(char *s)
 void
 set_attr(int a)
 {				/* set the attribute from the bits in a */
-    int i, b[32];
+    int i, b[10];
+    int use_sgr = 0;
 
     if (magic_cookie_glitch > 0) {
 	char_count += magic_cookie_glitch;
@@ -271,11 +244,17 @@ set_attr(int a)
 	put_mode(exit_attribute_mode);
 	return;
     }
-    for (i = 0; i < 31; i++) {
+    memset(b, 0, sizeof(b));
+    for (i = 0; i < 10; i++) {
 	b[i] = (a >> i) & 1;
+	if (b[i])
+	    use_sgr = 1;
+    }
+    if (use_sgr) {
+	tc_putp(TPARM_9(set_attributes,
+			b[1], b[2], b[3], b[4], b[5],
+			b[6], b[7], b[8], b[9]));
     }
-    tc_putp(TPARM_9(set_attributes, b[1], b[2], b[3], b[4], b[5],
-		    b[6], b[7], b[8], b[9]));
 }
 
 /*
@@ -285,11 +264,11 @@ set_attr(int a)
 */
 static void
 charset_sgr(
-	       struct test_list *t,
+	       TestList * t,
 	       int *state,
 	       int *ch)
 {
-    int i, j;
+    int i;
 
     if (!set_attributes) {
 	ptext("(sgr) Set-graphics-rendition is not defined.  ");
@@ -300,20 +279,31 @@ charset_sgr(
 	ptextln("(sgr0) Set-graphics-rendition-zero is not defined.");
 	/* go ahead and test anyway */
     }
-    ptext("Test video attributes (sgr)");
+    ptext("Test video attributes");
 
     for (i = 0; i < (int) (sizeof(alt_modes) / sizeof(struct mode_list));
 	 i++) {
 	put_crlf();
-	sprintf(temp, "%d %-20s", i, alt_modes[i].name);
+	sprintf(temp, "%2d %-20s", i, alt_modes[i].name);
 	put_str(temp);
 	set_attr(alt_modes[i].number);
 	sprintf(temp, "%s", alt_modes[i].name);
 	put_str(temp);
 	set_attr(0);
     }
+    put_crlf();
+    generic_done_message(t, state, ch);
+}
 
-    putln("\n\nDouble mode test");
+static void
+charset_sgr2(
+		TestList * t,
+		int *state,
+		int *ch)
+{
+    int i, j;
+
+    putln("Double mode test");
     for (i = 0; i <= 9; i++) {
 	sprintf(temp, " %2d ", mode_map[i]);
 	put_str(temp);
@@ -342,6 +332,49 @@ charset_sgr(
     generic_done_message(t, state, ch);
 }
 
+static void
+charset_italics(
+		   TestList * t,
+		   int *state,
+		   int *ch)
+{
+    putln("Testing italics");
+    if (enter_italics_mode && exit_italics_mode) {
+	put_mode(enter_italics_mode);
+	ptext(" This should be in ITALICS");
+	put_mode(exit_italics_mode);
+	put_crlf();
+    } else {
+	putln("your terminal description does not tell how to show italics");
+    }
+    generic_done_message(t, state, ch);
+}
+
+/*
+ * rmxx/smxx describes the ECMA-48 strikeout/crossed-out attributes, as an
+ * experimental feature of tmux.
+ */
+static void
+charset_crossed(
+		   TestList * t,
+		   int *state,
+		   int *ch)
+{
+    const char *smxx = safe_tgets("smxx");
+    const char *rmxx = safe_tgets("rmxx");
+    putln("Testing cross-out/strike-out (ncurses/tmux extension)");
+    if (VALID_STRING(smxx) && VALID_STRING(rmxx)) {
+	put_mode(smxx);
+	ptext(" This should be CROSSED-OUT");
+	put_mode(rmxx);
+	put_crlf();
+    } else {
+	putln("your terminal description does not tell how to cross-out text");
+    }
+    ptext("(smxx) (rmxx) ");
+    generic_done_message(t, state, ch);
+}
+
 /*
 **	test_one_attr(mode-number, begin-string, end-string)
 **
@@ -350,8 +383,8 @@ charset_sgr(
 static void
 test_one_attr(
 		 int n,
-		 char *begin_mode,
-		 char *end_mode)
+		 const char *begin_mode,
+		 const char *end_mode)
 {
     int i;
 
@@ -392,10 +425,11 @@ test_one_attr(
 */
 static void
 charset_attributes(
-		      struct test_list *t,
+		      TestList * t,
 		      int *state,
 		      int *ch)
 {
+    put_clear();
     putln("Test video attributes");
     test_one_attr(1, enter_standout_mode, exit_standout_mode);
     test_one_attr(2, enter_underline_mode, exit_underline_mode);
@@ -424,13 +458,13 @@ charset_attributes(
 */
 static void
 charset_smacs(
-		 struct test_list *t,
+		 TestList * t,
 		 int *state,
 		 int *ch)
 {
-    int i, c;
-
     if (enter_alt_charset_mode) {
+	int i, c;
+
 	put_clear();
 	ptextln("The following characters are available. (smacs) (rmacs)");
 	for (i = ' '; i <= '`'; i += 32) {
@@ -457,10 +491,9 @@ static void
 test_acs(
 	    int attr)
 {				/* alternate character set */
-    int i, j;
+    int i;
     char valid_glyph[GLYPHS];
     char acs_table[GLYPHS];
-    static unsigned char vt100[] = "`afgjklmnopqrstuvwxyz{|}~";
 
     line_count = 0;
     for (i = 0; i < GLYPHS; i++) {
@@ -468,10 +501,12 @@ test_acs(
 	acs_table[i] = (char) i;
     }
     if (acs_chars) {
-	sprintf(temp, "Alternate character set map: %s",
-		expand(acs_chars));
-	putln(temp);
+	putln("Alternate character set map:");
+	putln(expand(acs_chars));
+	put_crlf();
 	for (i = 0; acs_chars[i]; i += 2) {
+	    int j;
+
 	    if (acs_chars[i + 1] == 0) {
 		break;
 	    }
@@ -496,6 +531,8 @@ test_acs(
 	    }
 	}
     } else {
+	static unsigned char vt100[] = "`afgjklmnopqrstuvwxyz{|}~";
+
 	ptextln("acs_chars not defined (acsc)");
 	/* enable the VT-100 graphics characters (default) */
 	for (i = 0; vt100[i]; i++) {
@@ -505,7 +542,7 @@ test_acs(
     if (attr) {
 	set_attr(attr);
     }
-    _nc_init_acs();		/* puts 'ena_acs' and incidentally links acs_map[] */
+    put_mode(ena_acs);
     for (i = 0; glyph[i].name[0]; i++) {
 	if (valid_glyph[glyph[i].c]) {
 	    put_mode(enter_alt_charset_mode);
@@ -581,7 +618,7 @@ test_acs(
 */
 static void
 charset_bel(
-	       struct test_list *t,
+	       TestList * t,
 	       int *state,
 	       int *ch)
 {
@@ -602,7 +639,7 @@ charset_bel(
 */
 static void
 charset_flash(
-		 struct test_list *t,
+		 TestList * t,
 		 int *state,
 		 int *ch)
 {
@@ -623,7 +660,7 @@ charset_flash(
 */
 static void
 charset_civis(
-		 struct test_list *t,
+		 TestList * t,
 		 int *state,
 		 int *ch)
 {
@@ -647,7 +684,7 @@ charset_civis(
 */
 static void
 charset_cvvis(
-		 struct test_list *t,
+		 TestList * t,
 		 int *state,
 		 int *ch)
 {
@@ -671,7 +708,7 @@ charset_cvvis(
 */
 static void
 charset_cnorm(
-		 struct test_list *t,
+		 TestList * t,
 		 int *state,
 		 int *ch)
 {
@@ -693,13 +730,13 @@ charset_cnorm(
 */
 static void
 charset_enacs(
-		 struct test_list *t,
+		 TestList * t,
 		 int *state,
 		 int *ch)
 {
-    int c, i;
-
     if (enter_alt_charset_mode || acs_chars) {
+	int c, i;
+
 	c = 0;
 	while (1) {
 	    put_clear();
@@ -755,3 +792,27 @@ charset_can_test(void)
 	can_test(alt_modes[i].end_mode, FLAG_CAN_TEST);
     }
 }
+/* *INDENT-OFF* */
+
+TestList acs_test_list[] =
+{
+    MY_EDIT_MENU
+    {MENU_NEXT, 3, "bel",   0, "b) bel/flash", charset_bel, 0},
+    {MENU_NEXT, 3, "flash", 0, 0, charset_flash, 0},
+    {MENU_NEXT, 3, "civis", 0, "c) cursor appearance", charset_civis, 0},
+    {MENU_NEXT, 3, "cvvis", 0, 0, charset_cvvis, 0},
+    {MENU_NEXT, 3, "cnorm", 0, 0, charset_cnorm, 0},
+    {MENU_NEXT, 3, "hs",    0, "t) title/statusline", charset_hs, 0},
+    {MENU_NEXT, 3, "tsl) (fsl) (wsl", "hs", 0, charset_status, 0},
+    {MENU_NEXT, 3, "eslok", "hs", 0, charset_eslok, 0},
+    {MENU_NEXT, 3, "dsl", "hs", 0, charset_dsl, 0},
+    {MENU_NEXT, 0, "acsc) (enacs) (smacs) (rmacs", 0, "a) alternate character set", charset_enacs, 0},
+    {MENU_NEXT, 12, "smacs) (rmacs", 0, 0, charset_smacs, 0},
+    {MENU_NEXT, 12, "sgr) (sgr0", 0, 0, charset_attributes, 0},
+    {MENU_NEXT, 12, "sgr) (sgr0", "ma", 0, charset_sgr, 0},
+    {MENU_NEXT, 12, "sgr) (sgr0", "ma", 0, charset_sgr2, 0},
+    {MENU_NEXT, 3, "sitm) (ritm", 0, 0, charset_italics, 0},
+    {MENU_NEXT, 3, 0, 0, 0, charset_crossed, 0},
+    {MENU_LAST, 0, 0, 0, 0, 0, 0}
+};
+/* *INDENT-ON* */
