@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1991, 1997-2013,2015 Free Software Foundation, Inc.
+** Copyright (C) 1991, 1997-2017,2019 Free Software Foundation, Inc.
 **
 ** This file is part of TACK.
 **
@@ -19,7 +19,7 @@
 ** Boston, MA 02110-1301, USA
 */
 
-/* $Id: tack.h,v 1.37 2015/06/02 09:05:07 tom Exp $ */
+/* $Id: tack.h,v 1.74 2019/07/21 19:08:15 tom Exp $ */
 
 #ifndef NCURSES_TACK_H_incl
 #define NCURSES_TACK_H_incl 1
@@ -27,14 +27,12 @@
 /* terminfo action checker include file */
 
 #define MAJOR_VERSION 1
-#define MINOR_VERSION 7
-#define PATCH_VERSION 20150606
+#define MINOR_VERSION 8
+#define PATCH_VERSION 20190721
 
 #ifdef HAVE_CONFIG_H
 #include <ncurses_cfg.h>
 #else
-#define GCC_UNUSED		/*nothing */
-#define GCC_NORETURN		/*nothing */
 #define HAVE_GETTIMEOFDAY 1
 #define HAVE_SELECT 1
 #define HAVE_SYS_TIME_H 1
@@ -44,10 +42,22 @@
 #define BROKEN_LINKER 0
 #endif
 
+#ifndef GCC_NORETURN
+#define GCC_NORETURN		/*nothing */
+#endif
+
+#ifndef GCC_UNUSED
+#define GCC_UNUSED		/*nothing */
+#endif
+
 #ifndef HAVE_LONG_FILE_NAMES
 #define HAVE_LONG_FILE_NAMES 0
 #endif
 
+#ifndef NCURSES_CONST
+#define NCURSES_CONST const
+#endif
+
 #ifndef NO_LEAKS
 #define NO_LEAKS 0
 #endif
@@ -72,8 +82,18 @@
 #include <signal.h>		/* include before curses.h to work around glibc bug */
 
 #include <curses.h>
+
+#if defined(NCURSES_VERSION) && defined(HAVE_TERM_ENTRY_H)
 #include <term_entry.h>
-#include <nc_tparm.h>
+#define TACK_CAN_EDIT 1
+#else
+#define TACK_CAN_EDIT 0
+#include <term.h>
+#include <termios.h>
+#define TTY struct termios
+#define TERMIOS 1
+#define GET_TTY(fd, buf) tcgetattr(fd, buf)
+#endif
 
 #if USE_RCS_IDS
 #define MODULE_ID(id) static const char Ident[] = id;
@@ -87,30 +107,125 @@
 #define IGNORE_RC(func) (void) func
 #endif /* gcc workarounds */
 
-#if NO_LEAKS && defined(HAVE__NC_FREE_TIC)
-extern void _nc_free_tic(int) GCC_NORETURN;
+#ifndef down_half_line
+#define down_half_line 0	/* NetBSD bug */
+#endif
+#ifndef hue_lightness_saturation
+#define hue_lightness_saturation 0	/* NetBSD bug */
+#endif
+#ifndef plab_norm
+#define plab_norm 0		/* NetBSD bug */
+#endif
+#ifndef prtr_off
+#define prtr_off 0		/* NetBSD bug */
+#endif
+#ifndef prtr_on
+#define prtr_on 0		/* NetBSD bug */
+#endif
+
+#if NO_LEAKS
 extern void tack_edit_leaks(void);
 extern void tack_fun_leaks(void);
+#ifdef HAVE__NC_FREE_TINFO
+extern void _nc_free_tinfo(int) GCC_NORETURN;
+#endif
+#ifndef ExitProgram
 extern void ExitProgram(int) GCC_NORETURN;
+#endif
 #else
 #define ExitProgram(code) exit(code)
 #undef  NO_LEAKS
 #define NO_LEAKS 0
 #endif
 
+#ifdef NCURSES_VERSION
+#define ChangeTermInfo(name,value) name = value
+#else
+#define ChangeTermInfo(name,value)	/* nothing */
+#endif
+
 #define FreeIfNeeded(p) if (p) { free(p); p = 0; }
 
-#define CUR_TP      (&(cur_term->type))
+#include <tackgen.h>
+
+#ifdef DECL_CURSES_DATA_BOOLNAMES
+#undef boolnames
+extern char *boolnames[];
+extern size_t max_booleans;
+
+#undef numnames
+extern char *numnames[];
+extern size_t max_numbers;
+
+#undef strnames
+extern char *strnames[];
+extern size_t max_strings;
+
+#undef boolfnames
+extern char *boolfnames[];
+
+#undef numfnames
+extern char *numfnames[];
+
+#undef strfnames
+extern char *strfnames[];
+
+#endif
+
+#define ABSENT_STRING		(char *)0
+#define CANCELLED_STRING	(char *)(-1)
+#define VALID_STRING(s)  ((s) != CANCELLED_STRING && (s) != ABSENT_STRING)
+
+#if TACK_CAN_EDIT
+#define CUR_TP      ((TERMTYPE *)(cur_term))
+#define MAX_BOOLEAN BOOLCOUNT	/* NUM_BOOLEANS(CUR_TP) */
+#define MAX_NUMBERS NUMCOUNT	/* NUM_NUMBERS(CUR_TP) */
 #define MAX_STRINGS NUM_STRINGS(CUR_TP)
 #define STR_NAME(n) ExtStrname(CUR_TP,n,strnames)
+#elif defined(BOOLCOUNT)
+#define MAX_BOOLEAN BOOLCOUNT
+#define MAX_NUMBERS NUMCOUNT
+#define MAX_STRINGS STRCOUNT
+#define STR_NAME(n) strnames[n]
+#else
+#define MAX_BOOLEAN max_booleans
+#define MAX_NUMBERS max_numbers
+#define MAX_STRINGS max_strings
+#define STR_NAME(n) strnames[n]
+#endif
+
+typedef enum {
+    BOOLEAN,
+    NUMBER,
+    STRING
+} NAME_TYPE;
+
+/* see ncurses' nc_tparm.h */
+#define TPARM_ARG long
+#define TPARM_N(n) (TPARM_ARG)(n)
+
+#define TPARM_9(a,b,c,d,e,f,g,h,i,j) tparm(a,TPARM_N(b),TPARM_N(c),TPARM_N(d),TPARM_N(e),TPARM_N(f),TPARM_N(g),TPARM_N(h),TPARM_N(i),TPARM_N(j))
+#define TPARM_8(a,b,c,d,e,f,g,h,i) TPARM_9(a,b,c,d,e,f,g,h,i,0)
+#define TPARM_7(a,b,c,d,e,f,g,h) TPARM_8(a,b,c,d,e,f,g,h,0)
+#define TPARM_6(a,b,c,d,e,f,g) TPARM_7(a,b,c,d,e,f,g,0)
+#define TPARM_5(a,b,c,d,e,f) TPARM_6(a,b,c,d,e,f,0)
+#define TPARM_4(a,b,c,d,e) TPARM_5(a,b,c,d,e,0)
+#define TPARM_3(a,b,c,d) TPARM_4(a,b,c,d,0)
+#define TPARM_2(a,b,c) TPARM_3(a,b,c,0)
+#define TPARM_1(a,b) TPARM_2(a,b,0)
+#define TPARM_1(a,b) TPARM_2(a,b,0)
+#define TPARM_0(a) TPARM_1(a,0)
 
 #define UChar(c)    ((unsigned char)(c))
 
+#define NAME_SIZE 32
+#define TEMP_SIZE 1024
+
 extern FILE *log_fp;
 extern FILE *debug_fp;
 extern int debug_level;
-extern char temp[];
-extern char tty_basename[];
+extern char temp[TEMP_SIZE];
+extern char *tty_basename;
 extern char tty_shortname[];
 
 #if defined(__GNUC__) && defined(_FORTIFY_SOURCE)
@@ -194,22 +309,22 @@ extern int send_reset_init;
 #define MAX_SCAN 256
 
 /* translate mode default strings */
-struct default_string_list {
+typedef struct default_string_list {
     const char *name;		/* terminfo name */
     const char *value;		/* value of default string */
     int index;			/* index into the strfname[] array */
-};
+} DefaultStringList;
 
 #define TM_last 8
 extern struct default_string_list TM_string[TM_last];
 
 /* attribute structure definition */
-struct mode_list {
+typedef struct mode_list {
     const char *name;
     const char *begin_mode;
     const char *end_mode;
     int number;
-};
+} ModeList;
 
 extern const struct mode_list alt_modes[];
 extern const int mode_map[];
@@ -249,38 +364,42 @@ extern unsigned long tx_cps;	/* characters per second */
 	Menu control for tack.
 */
 
-struct test_results {
+struct test_list;
+typedef void (TestFunc) (struct test_list * t, int *state, int *ch);
+
+struct test_menu;
+typedef void (MenuFunc) (struct test_menu *);
+
+typedef struct test_results {
     struct test_results *next;	/* point to next entry */
     struct test_list *test;	/* Test which got these results */
     int reps;			/* repeat count */
     int delay;			/* delay times 10 */
-};
+} TestResults;
 
-struct test_list {
+typedef struct test_list {
     int flags;			/* Test description flags */
     int lines_needed;		/* Lines needed for test (0->no action) */
     const char *caps_done;	/* Caps shown in Done message */
     const char *caps_tested;	/* Other caps also being tested */
     const char *menu_entry;	/* Menu entry text (optional) */
-    /* Function that does testing */
-    void (*test_procedure) (struct test_list *, int *, int *);
+    TestFunc *test_procedure;	/* Function that does testing */
     struct test_menu *sub_menu;	/* Nested sub-menu */
-};
+} TestList;
 
-struct test_menu {
+typedef struct test_menu {
     int flags;			/* Menu feature flag */
     int default_action;		/* Default command if <cr> <lf> entered */
     const char *menu_text;	/* Describe this test_menu */
     const char *menu_title;	/* Title for the menu */
     const char *ident;		/* short menu name */
     const char *standard_tests;	/* Standard test text */
-    /* print current settings (optional) */
-    void (*menu_function) (struct test_menu *);
-    struct test_list *tests;	/* Pointer to the menu/function pairs */
-    struct test_list *resume_tests;	/* Standard test resume point */
+    MenuFunc *menu_function;	/* print current settings (optional) */
+    TestList *tests;		/* Pointer to the menu/function pairs */
+    TestList *resume_tests;	/* Standard test resume point */
     int resume_state;		/* resume state of test group */
     int resume_char;		/* resume ch of test group */
-};
+} TestMenu;
 
 /* menu flags */
 #define MENU_100c	0x00001a00	/* Augment 100% of columns */
@@ -317,7 +436,18 @@ struct test_menu {
 #define REQUEST_PROMPT 256
 
 /* tack.c */
-extern struct test_menu edit_menu;
+#if TACK_CAN_EDIT
+extern TestMenu edit_menu;
+#define MY_EDIT_MENU	{0, 0, 0, 0, "e) edit terminfo", 0, &edit_menu},
+#else
+#define MY_EDIT_MENU		/* nothing */
+#endif
+#ifdef DEBUG
+#define TACKMSG(p) p
+extern void TackMsg(const char *, ...) GCC_PRINTFLIKE(1,2);
+#else
+#define TACKMSG(p)		/* nothing */
+#endif
 extern void show_usage(char *);
 extern void print_version(void);
 
@@ -329,7 +459,6 @@ extern char *hex_expand_to(char *, int);
 extern char *print_expand(char *);
 extern int getchp(int);
 extern int getnext(int);
-extern int tc_putch(int);
 extern int tc_putp(const char *);
 extern int wait_here(void);
 extern void go_home(void);
@@ -355,70 +484,83 @@ extern void tt_putp(const char *);
 extern void tt_putparm(NCURSES_CONST char *, int, int, int);
 extern void tt_tputs(const char *, int);
 
+/* Solaris is out of step - humor it */
+#if defined(__EXTENSIONS__) && !defined(NCURSES_VERSION)
+#define TC_PUTCH char
+#else
+#define TC_PUTCH int
+#endif
+extern int tc_putch(TC_PUTCH);
+
 #define put_that(n) put_this((int) (n))
 
 /* control.c */
-extern struct test_list color_test_list[];
-extern char *liberated(char *);
+extern TestList color_test_list[];
+extern char *liberated(const char *);
 extern char txt_longer_augment[80];
 extern char txt_longer_test_time[80];
 extern char txt_shorter_augment[80];
 extern char txt_shorter_test_time[80];
 extern int msec_cost(const char *const, int);
-extern int skip_pad_test(struct test_list *, int *, int *, const char *);
+extern int skip_pad_test(TestList *, int *, int *, const char *);
 extern int sliding_scale(int, int, unsigned long);
 extern int still_testing(void);
 extern long event_time(int);
 extern void control_init(void);
-extern void dump_test_stats(struct test_list *, int *, int *);
+extern void dump_test_stats(TestList *, int *, int *);
 extern void event_start(int);
-extern void longer_augment(struct test_list *, int *, int *);
-extern void longer_test_time(struct test_list *, int *, int *);
-extern void pad_test_shutdown(struct test_list *, int);
+extern void longer_augment(TestList *, int *, int *);
+extern void longer_test_time(TestList *, int *, int *);
+extern void pad_test_shutdown(TestList *, int);
 extern void pad_test_startup(int);
 extern void page_loop(void);
 extern void set_augment_txt(void);
-extern void shorter_augment(struct test_list *, int *, int *);
-extern void shorter_test_time(struct test_list *, int *, int *);
+extern void shorter_augment(TestList *, int *, int *);
+extern void shorter_test_time(TestList *, int *, int *);
 
 /* charset.c */
-extern struct test_list acs_test_list[];
+extern TestList acs_test_list[];
 extern void set_attr(int);
 extern void eat_cookie(void);
-extern void put_mode(char *);
+extern void put_mode(const char *);
 
 /* crum.c */
-extern struct test_list crum_test_list[];
+extern TestList crum_test_list[];
 
 /* ansi.c */
-extern void tools_status(struct test_list *, int *, int *);
-extern void tools_charset(struct test_list *, int *, int *);
-extern void tools_sgr(struct test_list *, int *, int *);
+extern void tools_status(TestList *, int *, int *);
+extern void tools_charset(TestList *, int *, int *);
+extern void tools_sgr(TestList *, int *, int *);
 
 /* edit.c */
-extern struct test_menu change_pad_menu;
-extern struct test_list edit_test_list[];
-extern char *get_string_cap_byname(const char *, const char **);
+#if TACK_CAN_EDIT
+extern TestMenu change_pad_menu;
+extern TestList edit_test_list[];
+#define MY_PADS_MENU	{0, 0, 0, 0, "p) change padding", 0, &change_pad_menu},
+#else
+#define MY_PADS_MENU		/* nothing */
+#endif
+extern const char *get_string_cap_byname(const char *, const char **);
 extern int cap_match(const char *names, const char *cap);
 extern int get_string_cap_byvalue(const char *);
 extern int user_modified(void);
 extern void can_test(const char *, int);
 extern void cap_index(const char *, int *);
 extern void edit_init(void);
-extern void save_info(struct test_list *, int *, int *);
-extern void show_report(struct test_list *, int *, int *);
+extern void save_info(TestList *, int *, int *);
+extern void show_report(TestList *, int *, int *);
 
 /* fun.c */
-extern struct test_list funkey_test_list[];
-extern struct test_list printer_test_list[];
+extern TestList funkey_test_list[];
+extern TestList printer_test_list[];
 extern void enter_key(const char *, char *, char *);
 extern int tty_meta_prep(void);
-extern void tools_report(struct test_list *, int *, int *);
+extern void tools_report(TestList *, int *, int *);
 
 /* init.c */
+extern const char *safe_tgets(NCURSES_CONST char *);
 extern void reset_init(void);
 extern void display_basic(void);
-extern void put_name(const char *, const char *);
 extern void charset_can_test(void);
 extern void curses_setup(char *);
 extern void bye_kids(int);
@@ -444,28 +586,28 @@ extern void tty_set(void);
 
 /* menu.c */
 extern char prompt_string[80];	/* menu prompt storage */
-extern int subtest_menu(struct test_list *, int *, int *);
-extern struct test_list *augment_test;
-extern void generic_done_message(struct test_list *, int *, int *);
-extern void menu_can_scan(const struct test_menu *);
-extern void menu_clear_screen(struct test_list *, int *, int *);
-extern void menu_display(struct test_menu *, int *);
+extern int subtest_menu(TestList *, int *, int *);
+extern TestList *augment_test;
+extern void generic_done_message(TestList *, int *, int *);
+extern void menu_can_scan(const TestMenu *);
+extern void menu_clear_screen(TestList *, int *, int *);
+extern void menu_display(TestMenu *, int *);
 extern void menu_prompt(void);
-extern void menu_reset_init(struct test_list *, int *, int *);
-extern void pad_done_message(struct test_list *, int *, int *);
+extern void menu_reset_init(TestList *, int *, int *);
+extern void pad_done_message(TestList *, int *, int *);
 
 /* modes.c */
-extern struct test_list mode_test_list[];
+extern TestList mode_test_list[];
 
 /* pad.c */
-extern struct test_list pad_test_list[];
+extern TestList pad_test_list[];
 
 /* sync.c */
-extern struct test_menu sync_menu;
+extern TestMenu sync_menu;
 extern int tty_sync_error(void);
 extern void flush_input(void);
-extern void sync_handshake(struct test_list *, int *, int *);
-extern void sync_test(struct test_menu *);
+extern void sync_handshake(TestList *, int *, int *);
+extern void sync_test(TestMenu *);
 extern void verify_time(void);
 
 #endif /* NCURSES_TACK_H_incl */
